@@ -1,12 +1,15 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 
 use strict;
 use warnings FATAL => 'all';
 use File::Basename;
 
-use utf8;
+# Set the input (STDIN) as UTF8, but don't give warnings about unknown chars
 use open qw(:std :utf8); # http://stackoverflow.com/a/519359
-binmode STDOUT,':utf8';
+no warnings 'utf8';
+
+# Set the output to always be UTF8
+binmode STDOUT,':encoding(UTF-8)';
 
 my $remove_file_add_header    = 1;
 my $remove_file_delete_header = 1;
@@ -78,22 +81,7 @@ while (my $line = <>) {
 			$last_file_seen = $file_2;
 		}
 
-		# If they're the same it's a modify
-		if ($file_1 eq $file_2) {
-			print "modified: $file_1\n";
-		# If the first is /dev/null it's a new file
-		} elsif ($file_1 eq "/dev/null") {
-			print "added: $file_2\n";
-		# If the second is /dev/null it's a deletion
-		} elsif ($file_2 eq "/dev/null") {
-			print "deleted: $file_1\n";
-		# If the files aren't the same it's a rename
-		} elsif ($file_1 ne $file_2) {
-			print "renamed: $file_1 to $file_2\n";
-		# Something we haven't thought of yet
-		} else {
-			print "$file_1 -> $file_2\n";
-		}
+		print file_change_string($file_1,$file_2) . "\n";
 
 		# Print out the bottom horizontal line of the header
 		print horizontal_rule($horizontal_color);
@@ -126,11 +114,12 @@ while (my $line = <>) {
 	######################################
 	} elsif ($remove_file_delete_header && $line =~ /^${ansi_color_regex}deleted file mode/) {
 		# Don't print the line (i.e. remove it from the output);
-	######################################
-	# Look for binary file changes
-	######################################
-	} elsif ($line =~ /Binary files \w\/(.+?) and \w\/(.+?) differ/) {
-		print "${horizontal_color}modified: $2 (binary)\n";
+	################################
+	# Look for binary file changes #
+	################################
+	} elsif ($line =~ /^Binary files (\w\/)?(.+?) and (\w\/)?(.+?) differ/) {
+		my $change = file_change_string($2,$4);
+		print "$horizontal_color$change (binary)\n";
 		print horizontal_rule($horizontal_color);
 	#####################################################
 	# Check if we're changing the permissions of a file #
@@ -280,6 +269,8 @@ sub get_less_charset {
 	foreach (@less_char_vars) {
 		return $ENV{$_} if defined $ENV{$_};
 	}
+
+	return "";
 }
 
 sub should_print_unicode {
@@ -287,8 +278,14 @@ sub should_print_unicode {
 		# Always print unicode chars if we're not piping stuff, e.g. to less(1)
 		return 1;
 	}
+
 	# Otherwise, assume we're piping to less(1)
-	return get_less_charset() =~ /utf-?8/i;
+	my $less_charset = get_less_charset();
+	if ($less_charset =~ /utf-?8/i) {
+		return 1;
+	}
+
+	return 0;
 }
 
 # Return git config as a hash
@@ -405,3 +402,25 @@ sub horizontal_rule {
 
 	return $ret;
 }
+
+sub file_change_string {
+	my $file_1 = shift();
+	my $file_2 = shift();
+
+	# If they're the same it's a modify
+	if ($file_1 eq $file_2) {
+		return "modified: $file_1";
+	# If the first is /dev/null it's a new file
+	} elsif ($file_1 eq "/dev/null") {
+		return "added: $file_2";
+	# If the second is /dev/null it's a deletion
+	} elsif ($file_2 eq "/dev/null") {
+		return "deleted: $file_1";
+	# If the files aren't the same it's a rename
+	} elsif ($file_1 ne $file_2) {
+		return "renamed: $file_1 to $file_2";
+	# Something we haven't thought of yet
+	} else {
+		return "$file_1 -> $file_2";
+	}
+}
